@@ -7,7 +7,6 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"go/build"
 	"io"
 	"io/fs"
 	"net/http"
@@ -381,9 +380,8 @@ func newConfig(options ...configOption) (*Config, error) {
 			recursive: true,
 		},
 		reAdd: reAddCmdConfig{
-			exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
-			recursive: true,
+			exclude: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
+			include: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
 		},
 		state: stateCmdConfig{
 			data: stateDataCmdConfig{
@@ -436,6 +434,7 @@ func newConfig(options ...configOption) (*Config, error) {
 		"ioreg":                    c.ioregTemplateFunc,
 		"joinPath":                 c.joinPathTemplateFunc,
 		"keepassxc":                c.keepassxcTemplateFunc,
+		"keepassxcAttachment":      c.keepassxcAttachmentTemplateFunc,
 		"keepassxcAttribute":       c.keepassxcAttributeTemplateFunc,
 		"keyring":                  c.keyringTemplateFunc,
 		"lastpass":                 c.lastpassTemplateFunc,
@@ -733,18 +732,13 @@ func (c *Config) createConfigFile(filename chezmoi.RelPath, data []byte) ([]byte
 	funcMap := make(template.FuncMap)
 	chezmoi.RecursiveMerge(funcMap, c.templateFuncs)
 	initTemplateFuncs := map[string]interface{}{
+		"exit":          c.exitInitTemplateFunc,
 		"promptBool":    c.promptBoolInitTemplateFunc,
 		"promptInt":     c.promptIntInitTemplateFunc,
 		"promptString":  c.promptStringInitTemplateFunc,
 		"stdinIsATTY":   c.stdinIsATTYInitTemplateFunc,
 		"writeToStdout": c.writeToStdout,
 	}
-	for _, releaseTag := range build.Default.ReleaseTags {
-		if releaseTag == "go1.17" {
-			initTemplateFuncs["exit"] = c.exitInitTemplateFunc
-			break
-		}
-	}
 	chezmoi.RecursiveMerge(funcMap, initTemplateFuncs)
 
 	t, err := template.New(filename.String()).Funcs(funcMap).Parse(string(data))
@@ -1341,6 +1335,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		c.newExecuteTemplateCmd(),
 		c.newForgetCmd(),
 		c.newGitCmd(),
+		c.newIgnoredCmd(),
 		c.newImportCmd(),
 		c.newInitCmd(),
 		c.newInternalTestCmd(),
@@ -1369,6 +1364,23 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 	return rootCmd, nil
 }
 
+// newDiffSystem returns a system that logs all changes to s to w using
+// diff.command if set or the builtin git diff otherwise.
+func (c *Config) newDiffSystem(s chezmoi.System, w io.Writer, dirAbsPath chezmoi.AbsPath) chezmoi.System {
+	if c.Diff.useBuiltinDiff || c.Diff.Command == "" {
+		options := &chezmoi.GitDiffSystemOptions{
+			Color:   c.Color.Value(c.colorAutoFunc),
+			Include: c.Diff.include.Sub(c.Diff.Exclude),
+			Reverse: c.Diff.Reverse,
+		}
+		return chezmoi.NewGitDiffSystem(s, w, dirAbsPath, options)
+	}
+	options := &chezmoi.ExternalDiffSystemOptions{
+		Reverse: c.Diff.Reverse,
+	}
+	return chezmoi.NewExternalDiffSystem(s, c.Diff.Command, c.Diff.Args, c.DestDirAbsPath, options)
+}
+
 // newSourceState returns a new SourceState with options.
 func (c *Config) newSourceState(
 	ctx context.Context, options ...chezmoi.SourceStateOption,
@@ -1635,14 +1647,8 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 		c.destSystem = chezmoi.NewDryRunSystem(c.destSystem)
 	}
 	if c.Verbose {
-		c.sourceSystem = chezmoi.NewGitDiffSystem(c.sourceSystem, c.stdout, c.SourceDirAbsPath, &chezmoi.GitDiffSystemOptions{
-			Color:   color,
-			Include: c.Diff.include.Sub(c.Diff.Exclude),
-		})
-		c.destSystem = chezmoi.NewGitDiffSystem(c.destSystem, c.stdout, c.DestDirAbsPath, &chezmoi.GitDiffSystemOptions{
-			Color:   color,
-			Include: c.Diff.include.Sub(c.Diff.Exclude),
-		})
+		c.sourceSystem = c.newDiffSystem(c.sourceSystem, c.stdout, c.SourceDirAbsPath)
+		c.destSystem = c.newDiffSystem(c.destSystem, c.stdout, c.DestDirAbsPath)
 	}
 
 	if err := c.setEncryption(); err != nil {
